@@ -30,13 +30,11 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include <bonobo/bonobo-main.h>
 #include <libgnomevfs/gnome-vfs.h>
-#include <libgnomeui/gnome-stock-icons.h>
-#include <libgnome/libgnome.h>
 #include <glade/glade.h>
-#include <gconf/gconf-client.h>
 #include <string.h>
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-url.h>
 
 #include "yelp-bookmarks.h"
 #include "yelp-db-pager.h"
@@ -48,6 +46,7 @@
 #include "yelp-toc-pager.h"
 #include "yelp-window.h"
 #include "yelp-print.h"
+#include "yelp-debug.h"
 
 #ifdef ENABLE_MAN
 #include "yelp-man-pager.h"
@@ -60,12 +59,6 @@
 #include "gtkentryaction.h"
 #endif
 
-#ifdef YELP_DEBUG
-#define d(x) x
-#else
-#define d(x)
-#endif
-
 #define YELP_CONFIG_WIDTH          "/yelp/Geometry/width"
 #define YELP_CONFIG_HEIGHT         "/yelp/Geometry/height"
 #define YELP_CONFIG_WIDTH_DEFAULT  "600"
@@ -82,6 +75,9 @@ typedef struct {
     YelpDocInfo *doc_info;
 
     gchar *frag_id;
+    GtkWidget *menu_entry;
+    YelpWindow *window;
+    gint callback;
 
     gchar *page_title;
     gchar *frag_title;
@@ -155,6 +151,10 @@ static void        tree_drag_data_get_cb          (GtkWidget         *widget,
 						   guint              info,
 						   guint32            time,
 						   YelpWindow        *window);
+static void        tree_row_expand_cb             (GtkTreeView       *view,
+						   GtkTreePath       *path,
+						   GtkTreeViewColumn *column,
+						   YelpWindow        *window);
 
 /** UIManager Callbacks **/
 static void    window_add_widget        (GtkUIManager *ui_manager,
@@ -196,6 +196,10 @@ static void               history_step_back      (YelpWindow       *window);
 static YelpHistoryEntry * history_pop_back       (YelpWindow       *window);
 static YelpHistoryEntry * history_pop_forward    (YelpWindow       *window);
 static void               history_entry_free     (YelpHistoryEntry *entry);
+static void               history_back_to        (GtkMenuItem      *menuitem,
+						  YelpHistoryEntry *entry);
+static void               history_forward_to     (GtkMenuItem      *menuitem,
+						  YelpHistoryEntry *entry);
 
 static void               load_data_free         (YelpLoadData     *data);
 
@@ -266,6 +270,8 @@ struct _YelpWindowPriv {
     gchar          *current_frag;
     GSList         *history_back;
     GSList         *history_forward;
+    GtkWidget      *back_menu;
+    GtkWidget      *forward_menu;
 
     /* Callbacks and Idles */
     gulong          start_handler;
@@ -450,7 +456,7 @@ static const GtkActionEntry entries[] = {
       "F1", 
       N_("Help On this application"),
       G_CALLBACK (window_help_contents_cb) },
-    { "About", GNOME_STOCK_ABOUT,
+    { "About", GTK_STOCK_ABOUT,
       N_("_About"),
       NULL,
       NULL,
@@ -571,6 +577,7 @@ history_push_back (YelpWindow *window)
     YelpWindowPriv   *priv;
     YelpHistoryEntry *entry;
     GtkAction        *action;
+    gchar            *title;
 
     g_return_if_fail (YELP_IS_WINDOW (window));
     g_return_if_fail (window->priv->current_doc != NULL);
@@ -587,6 +594,27 @@ history_push_back (YelpWindow *window)
     action = gtk_action_group_get_action (priv->action_group, "GoBack");
     if (action)
 	g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
+    title = (gchar *) gtk_window_get_title (GTK_WINDOW (window));
+
+    if (g_str_equal (title, "Loading..."))
+	entry->page_title = g_strdup ("Unknown Page");
+    else
+	entry->page_title = g_strdup (title);
+
+    entry->window = window;
+    
+    entry->menu_entry = gtk_menu_item_new_with_label (entry->page_title);
+    g_object_ref (entry->menu_entry);
+
+    entry->callback = g_signal_connect (G_OBJECT (entry->menu_entry), 
+		      "activate", 
+		      G_CALLBACK (history_back_to),
+		      entry);
+		      
+
+    gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->back_menu), 
+			    entry->menu_entry);
+    gtk_widget_show (entry->menu_entry);
 }
 
 static void
@@ -595,6 +623,7 @@ history_push_forward (YelpWindow *window)
     YelpWindowPriv   *priv;
     YelpHistoryEntry *entry;
     GtkAction        *action;
+    gchar            *title;
 
     g_return_if_fail (YELP_IS_WINDOW (window));
     g_return_if_fail (window->priv->current_doc != NULL);
@@ -611,6 +640,27 @@ history_push_forward (YelpWindow *window)
     action = gtk_action_group_get_action (priv->action_group, "GoForward");
     if (action)
 	g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
+
+    title = (gchar *) gtk_window_get_title (GTK_WINDOW (window));
+
+    if (g_str_equal (title, "Loading..."))
+	entry->page_title = g_strdup ("Unknown Page");
+    else
+	entry->page_title = g_strdup (title);
+
+    entry->window = window;
+    entry->menu_entry = gtk_menu_item_new_with_label (entry->page_title);
+    g_object_ref (entry->menu_entry);
+
+    entry->callback = g_signal_connect (G_OBJECT (entry->menu_entry), 
+		      "activate", 
+		      G_CALLBACK (history_forward_to),
+		      entry);
+		      
+
+    gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->forward_menu), 
+			    entry->menu_entry);
+    gtk_widget_show (entry->menu_entry);
 }
 
 static void
@@ -726,9 +776,112 @@ history_entry_free (YelpHistoryEntry *entry)
     g_free (entry->page_title);
     g_free (entry->frag_title);
 
+    if (entry->menu_entry) {
+	gtk_widget_destroy (entry->menu_entry);
+    }
+
     g_free (entry);
 }
 
+static void 
+history_back_to (GtkMenuItem *menuitem, YelpHistoryEntry *entry)
+{
+    YelpWindow       *window;
+    YelpWindowPriv   *priv;
+    YelpHistoryEntry *latest;
+
+    window = entry->window;
+
+    priv = window->priv;
+
+    latest = history_pop_back (window);
+    history_push_forward (window);
+
+    while (!g_str_equal (latest->page_title, entry->page_title)) {
+	priv->history_forward = g_slist_prepend (priv->history_forward, latest);
+	gtk_container_remove (GTK_CONTAINER (priv->back_menu), 
+			      latest->menu_entry);
+	
+	g_signal_handler_disconnect (latest->menu_entry, latest->callback);
+	latest->callback = g_signal_connect (G_OBJECT (latest->menu_entry), 
+					     "activate", 
+					     G_CALLBACK (history_forward_to),
+					     latest);
+	
+	
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->forward_menu), 
+				latest->menu_entry);
+	gtk_widget_show (latest->menu_entry);
+
+	
+	latest = history_pop_back (window);
+    }
+    if (latest->menu_entry) {
+	gtk_widget_destroy (latest->menu_entry);
+	latest->menu_entry = NULL;
+    }
+
+    if (priv->current_doc)
+	yelp_doc_info_unref (priv->current_doc);
+    if (priv->current_frag)
+	g_free (priv->current_frag);
+
+    priv->current_doc  = yelp_doc_info_ref (entry->doc_info);
+    priv->current_frag = g_strdup (entry->frag_id);
+
+    window_do_load (window, entry->doc_info, entry->frag_id);
+
+}
+
+static void
+history_forward_to (GtkMenuItem *menuitem, YelpHistoryEntry *entry)
+{
+    YelpWindow       *window;
+    YelpWindowPriv   *priv;
+    YelpHistoryEntry *latest;
+
+    window = entry->window;
+
+    priv = window->priv;
+
+    latest = history_pop_forward (window);
+    history_push_back (window);
+
+    while (!g_str_equal (latest->page_title, entry->page_title)) {
+	priv->history_back = g_slist_prepend (priv->history_back, latest);
+	gtk_container_remove (GTK_CONTAINER (priv->forward_menu), 
+			      latest->menu_entry);
+	
+	g_signal_handler_disconnect (latest->menu_entry, latest->callback);
+	latest->callback = g_signal_connect (G_OBJECT (latest->menu_entry), 
+					     "activate", 
+					     G_CALLBACK (history_back_to),
+					     latest);
+
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->back_menu), 
+				latest->menu_entry);
+	gtk_widget_show (latest->menu_entry);
+
+	
+	latest = history_pop_forward (window);
+    }
+    if (latest->menu_entry) {
+	gtk_widget_destroy (latest->menu_entry);
+	latest->menu_entry = NULL;
+    }
+
+    if (priv->current_doc)
+	yelp_doc_info_unref (priv->current_doc);
+    if (priv->current_frag)
+	g_free (priv->current_frag);
+
+    priv->current_doc  = yelp_doc_info_ref (entry->doc_info);
+    priv->current_frag = g_strdup (entry->frag_id);
+
+    window_do_load (window, entry->doc_info, entry->frag_id);
+
+}
+
 static void
 load_data_free (YelpLoadData *data)
 {
@@ -763,8 +916,8 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
     } else {
 	real_uri = g_strdup (uri);
     }
-    d (g_print ("yelp_window_load\n"));
-    d (g_print ("  uri = \"%s\"\n", uri));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  uri = \"%s\"\n", uri);
 
     if (!real_uri) {
 	GError *error = NULL;
@@ -852,7 +1005,7 @@ window_do_load (YelpWindow  *window,
     g_return_if_fail (YELP_IS_WINDOW (window));
     g_return_if_fail (doc_info != NULL);
 
-    d (g_print ("window_do_laod\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     action = gtk_action_group_get_action (window->priv->action_group, 
 					  "PrintDocument");
@@ -1024,6 +1177,11 @@ window_populate (YelpWindow *window)
     GError            *error = NULL;
     GtkAction         *action;
     GtkTreeSelection  *selection;
+    GtkWidget         *toolbar;
+    GtkWidget         *toolitem;
+    GtkWidget         *b_proxy;
+    GtkWidget         *f_proxy;
+
 
     priv = window->priv;
 
@@ -1037,6 +1195,27 @@ window_populate (YelpWindow *window)
     gtk_action_group_add_actions (priv->action_group,
 				  entries, G_N_ELEMENTS (entries),
 				  window);
+
+    b_proxy = GTK_WIDGET (gtk_menu_tool_button_new (NULL, NULL));
+
+    priv->back_menu = gtk_menu_new ();
+
+    priv->forward_menu = gtk_menu_new ();
+
+    f_proxy = GTK_WIDGET (gtk_menu_tool_button_new (NULL, NULL));
+
+    gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (b_proxy), 
+				   priv->back_menu);
+
+    action = gtk_action_group_get_action(priv->action_group, "GoBack");
+
+    gtk_action_connect_proxy (action, b_proxy);
+
+    action = gtk_action_group_get_action (priv->action_group, "GoForward");
+    gtk_action_connect_proxy (action, f_proxy);
+
+    gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (f_proxy), 
+				   priv->forward_menu);
     
 #ifdef ENABLE_SEARCH
     action =  gtk_entry_action_new ("Search",
@@ -1074,8 +1253,6 @@ window_populate (YelpWindow *window)
 
     yelp_bookmarks_register (window);
 
-    gtk_ui_manager_ensure_update (priv->ui_manager);
-
     action = gtk_action_group_get_action (priv->action_group, "GoBack");
     if (action)
 	g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
@@ -1083,6 +1260,18 @@ window_populate (YelpWindow *window)
     if (action)
 	g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
 
+    gtk_ui_manager_ensure_update (priv->ui_manager);
+
+    toolbar = gtk_ui_manager_get_widget(priv->ui_manager, "ui/tools");
+
+    gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (f_proxy), 0);
+    gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (b_proxy), 0);
+
+    /*To keep the seperator but get rid of the extra widget*/
+    toolitem = gtk_ui_manager_get_widget(priv->ui_manager, "ui/tools/GoFor");
+
+    gtk_container_remove (GTK_CONTAINER (toolbar), toolitem);
+
     priv->popup = gtk_ui_manager_get_widget(priv->ui_manager, "ui/main_popup");
     priv->maillink = gtk_ui_manager_get_widget(priv->ui_manager, "ui/mail_popup");
     priv->merge_id = gtk_ui_manager_new_merge_id (priv->ui_manager);
@@ -1126,6 +1315,9 @@ window_populate (YelpWindow *window)
     g_signal_connect (selection, "changed",
 		      G_CALLBACK (tree_selection_changed_cb),
 		      window);
+    g_signal_connect (priv->side_sects, "row-activated",
+		      G_CALLBACK (tree_row_expand_cb),
+		      window);
     g_signal_connect (priv->side_sects, "drag_data_get",
 		      G_CALLBACK (tree_drag_data_get_cb),
 		      window);
@@ -1350,6 +1542,7 @@ window_do_load_pager (YelpWindow  *window,
     startnow = FALSE;
 
     state = yelp_pager_get_state (pager);
+    debug_print (DB_DEBUG, "pager state=%d\n", state);
     switch (state) {
     case YELP_PAGER_STATE_ERROR:
 	error = yelp_pager_get_error (pager);
@@ -1589,10 +1782,10 @@ window_handle_page (YelpWindow   *window,
     priv = window->priv;
     window_disconnect (window);
 
-    d (g_print ("window_handle_page\n"));
-    d (g_print ("  page->page_id  = \"%s\"\n", page->page_id));
-    d (g_print ("  page->title    = \"%s\"\n", page->title));
-    d (g_print ("  page->contents = %i bytes\n", strlen (page->contents)));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  page->page_id  = \"%s\"\n", page->page_id);
+    debug_print (DB_ARG, "  page->title    = \"%s\"\n", page->title);
+    debug_print (DB_ARG, "  page->contents = %i bytes\n", strlen (page->contents));
 
     model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->side_sects));
     pager = yelp_doc_info_get_pager (priv->current_doc);
@@ -1613,12 +1806,17 @@ window_handle_page (YelpWindow   *window,
 	    if (yelp_pager_page_contains_frag (pager,
 					       id,
 					       priv->current_frag)) {
-		GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
-
-		gtk_tree_view_expand_to_path (GTK_TREE_VIEW (priv->side_sects),
-					      path);
+		GtkTreePath *path = NULL;
+		GtkTreeIter parent;
+		if (gtk_tree_model_iter_parent (model, &parent, &iter)) {
+		    path = gtk_tree_model_get_path (model, &parent);
+		    gtk_tree_view_expand_to_path (GTK_TREE_VIEW (priv->side_sects),
+						  path);
+		    gtk_tree_path_free(path);
+		}
+		path = gtk_tree_model_get_path (model, &iter);
 		gtk_tree_selection_select_path (selection, path);
-
+		
 		gtk_tree_path_free (path);
 		g_free (id);
 		break;
@@ -1665,12 +1863,12 @@ window_handle_page (YelpWindow   *window,
 				 priv->current_frag,
 				 YELP_URI_TYPE_FILE);
 
-    d (g_print ("  uri            = %s\n", uri));
+    debug_print (DB_ARG, "  uri            = %s\n", uri);
 
     yelp_html_set_base_uri (priv->html_view, uri);
     yelp_html_open_stream (priv->html_view, "application/xhtml+xml");
 
-    priv->idle_write = gtk_idle_add ((GtkFunction) idle_write, context);
+    priv->idle_write = g_idle_add ((GtkFunction) idle_write, context);
 
     /*
     if (gnome_vfs_uri_get_fragment_identifier (uri->uri)) {
@@ -1687,7 +1885,7 @@ static void
 window_disconnect (YelpWindow *window)
 {
     YelpWindowPriv *priv;
-    YelpPager      *pager;
+    YelpPager      *pager = NULL;
 
     g_return_if_fail (YELP_IS_WINDOW (window));
 
@@ -1733,7 +1931,7 @@ window_disconnect (YelpWindow *window)
 	}
     }
     if (priv && priv->idle_write) {
-	gtk_idle_remove (priv->idle_write);
+	g_source_remove (priv->idle_write);
 	priv->idle_write = 0;
     }
 }
@@ -1767,8 +1965,8 @@ pager_start_cb (YelpPager   *pager,
     page_id = yelp_pager_resolve_frag (pager,
 				       window->priv->current_frag);
 
-    d (g_print ("pager_start_cb\n"));
-    d (g_print ("  page_id=\"%s\"\n", page_id));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  page_id=\"%s\"\n", page_id);
 
     window_set_sections (window,
 			 yelp_pager_get_sections (pager));
@@ -1796,8 +1994,8 @@ pager_page_cb (YelpPager *pager,
     YelpWindow  *window = YELP_WINDOW (user_data);
     YelpPage    *page;
 
-    d (g_print ("pager_page_cb\n"));
-    d (g_print ("  page_id=\"%s\"\n", page_id));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  page_id=\"%s\"\n", page_id);
 
     if (yelp_pager_page_contains_frag (pager,
 				       page_id,
@@ -1825,7 +2023,7 @@ pager_error_cb (YelpPager   *pager,
     YelpWindow *window = YELP_WINDOW (user_data);
     GError *error = yelp_pager_get_error (pager);
 
-    d (g_print ("pager_error_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     window_disconnect (window);
     window_error (window, error, TRUE);
@@ -1838,7 +2036,7 @@ pager_cancel_cb (YelpPager   *pager,
 		gpointer     user_data)
 {
     YelpWindow *window = YELP_WINDOW (user_data);
-    d (g_print ("pager_cancel_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     window_disconnect (window);
 }
@@ -1849,18 +2047,26 @@ pager_finish_cb (YelpPager   *pager,
 {
     GError *error = NULL;
     YelpWindow  *window = YELP_WINDOW (user_data);
+    const gchar *page_id;
+    
+    page_id = yelp_pager_resolve_frag (pager,
+				       window->priv->current_frag);
 
-    d (g_print ("pager_finish_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
+    
+    if (!page_id && window->priv->current_frag &&
+	strcmp (window->priv->current_frag, "")) {
 
-    window_disconnect (window);
+	window_disconnect (window);
 
-    g_set_error (&error, YELP_ERROR, YELP_ERROR_NO_PAGE,
-		 _("The section ‘%s’ does not exist in this document. "
-		   "If you were directed to this section from a Help "
-		   "button in an application, please report this to "
-		   "the maintainers of that application."),
-		 window->priv->current_frag);
-    window_error (window, error, TRUE);
+	g_set_error (&error, YELP_ERROR, YELP_ERROR_NO_PAGE,
+		     _("The section ‘%s’ does not exist in this document. "
+		       "If you were directed to this section from a Help "
+		       "button in an application, please report this to "
+		       "the maintainers of that application."),
+		     window->priv->current_frag);
+	window_error (window, error, TRUE);
+    }
 
     /* FIXME: Remove the URI from the history and go back */
 }
@@ -1875,8 +2081,8 @@ html_uri_selected_cb (YelpHtml  *html,
 {
     YelpWindow *window = YELP_WINDOW (user_data);
 
-    d (g_print ("html_uri_selected_cb\n"));
-    d (g_print ("  uri = \"%s\"\n", uri));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  uri = \"%s\"\n", uri);
 
     if (!handled) {
 	yelp_window_load (window, uri);
@@ -1983,7 +2189,7 @@ tree_drag_data_get_cb (GtkWidget         *widget,
 
     g_return_if_fail (YELP_IS_WINDOW (window));
 
-    d (g_print ("tree_drag_data_get_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     priv = window->priv;
 
@@ -2015,6 +2221,19 @@ tree_drag_data_get_cb (GtkWidget         *widget,
     g_free (uri);
 }
 
+void
+tree_row_expand_cb (GtkTreeView *view, GtkTreePath *path,
+		    GtkTreeViewColumn *column, YelpWindow *window)
+{
+	if (gtk_tree_view_row_expanded (view, path)) {
+		gtk_tree_view_collapse_row (view, path);
+	} else {
+		gtk_tree_view_expand_to_path (view, path);
+	}
+
+}
+
+
 /** UIManager Callbacks *******************************************************/
 
 static void
@@ -2045,7 +2264,7 @@ typedef struct {
 static void
 print_disconnect (PrintStruct *data)
 {
-    d(g_print ("print disconnect\n"));
+    debug_print (DB_FUNCTION, "entering\n");
     if (data->page_handler) {
 	g_signal_handler_disconnect (data->pager,
 				     data->page_handler);
@@ -2076,8 +2295,8 @@ print_pager_page_cb (YelpPager *pager,
 {
     PrintStruct *data = user_data;
     YelpPage    *page;
-    d (g_print ("print_pager_page_cb\n"));
-    d (g_print ("  page_id=\"%s\"\n", page_id));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  page_id=\"%s\"\n", page_id);
 
     page = (YelpPage *) yelp_pager_get_page (pager, page_id);
 
@@ -2087,7 +2306,7 @@ print_pager_page_cb (YelpPager *pager,
 	int length, offset;
 	char *uri;
 	GtkWidget *vbox = gtk_vbox_new (FALSE, FALSE);
-	d(g_print (page->contents));
+	debug_print (DB_DEBUG, page->contents);
 
 	gtk_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	html = yelp_html_new ();
@@ -2103,14 +2322,14 @@ print_pager_page_cb (YelpPager *pager,
 				     page_id,
 				     YELP_URI_TYPE_FILE);
 
-	d (g_print ("  uri            = %s\n", uri));
+	debug_print (DB_ARG, "  uri            = %s\n", uri);
 
 	yelp_html_set_base_uri (html, uri);
 	g_free (uri);
 
 	yelp_html_open_stream (html, "application/xhtml+xml");
 	for (length = strlen (page->contents), offset = 0; length > 0; length -= BUFFER_SIZE, offset += BUFFER_SIZE) {
-	    d(g_print ("data: %.*s\n", MIN (length, BUFFER_SIZE), page->contents + offset));
+	    debug_print (DB_DEBUG, "data: %.*s\n", MIN (length, BUFFER_SIZE), page->contents + offset);
 	    yelp_html_write (html, page->contents + offset, MIN (length, BUFFER_SIZE));
 	}
 	yelp_html_close (html);
@@ -2129,7 +2348,7 @@ print_pager_error_cb (YelpPager   *pager,
     PrintStruct *data = user_data;
     /*     GError *error = yelp_pager_get_error (pager);*/
 
-    d (g_print ("print_pager_error_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     print_disconnect (data);
 }
@@ -2139,7 +2358,7 @@ print_pager_cancel_cb (YelpPager   *pager,
 		       gpointer     user_data)
 {
     PrintStruct *data = user_data;
-    d (g_print ("print_pager_cancel_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     print_disconnect (data);
 }
@@ -2150,7 +2369,7 @@ print_pager_finish_cb (YelpPager   *pager,
 {
     PrintStruct *data = user_data;
 
-    d (g_print ("print_pager_finish_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     print_disconnect (data);
 }
@@ -2230,7 +2449,7 @@ window_print_page_cb (GtkAction *action, YelpWindow *window)
 	
 	yelp_html_open_stream (html, "application/xhtml+xml");
 	for (length = strlen (page->contents), offset = 0; length > 0; length -= BUFFER_SIZE, offset += BUFFER_SIZE) {
-	    d (g_print ("data: %.*s\n", MIN (length, BUFFER_SIZE), page->contents + offset));
+	    debug_print (DB_DEBUG, "data: %.*s\n", MIN (length, BUFFER_SIZE), page->contents + offset);
 	    yelp_html_write (html, page->contents + offset, MIN (length, BUFFER_SIZE));
 	}
 	yelp_html_close (html);
@@ -2401,7 +2620,7 @@ window_find_cb (GtkAction *action, YelpWindow *window)
 
     priv = window->priv;
 
-    d (g_print ("window_find_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     gtk_widget_show_all (priv->find_bar);
     gtk_widget_grab_focus (priv->find_entry);
@@ -2423,7 +2642,7 @@ window_reload_cb (GtkAction *action, YelpWindow *window)
 
     g_return_if_fail (YELP_IS_WINDOW (window));
 
-    d (g_print ("window_reload_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     if (window->priv->current_doc) {
 	YelpLoadData *data;
@@ -2448,16 +2667,7 @@ window_reload_cb (GtkAction *action, YelpWindow *window)
 static void
 window_enable_cursor_cb (GtkAction *action, YelpWindow *window)
 {
-    gboolean cursor;
-    GConfClient *gconf_client = gconf_client_get_default ();
-
-    cursor = gconf_client_get_bool (gconf_client,
-				    "/apps/yelp/use_caret",
-				    NULL);
-    gconf_client_set_bool (gconf_client,
-			   "/apps/yelp/use_caret",
-			   !cursor,
-			   NULL);
+    yelp_settings_toggle_caret ();
 }
 
 static gboolean
@@ -2529,7 +2739,7 @@ window_go_forward_cb (GtkAction *action, YelpWindow *window)
 static void
 window_go_home_cb (GtkAction *action, YelpWindow *window)
 {
-    d (g_print ("window_go_home_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
     g_return_if_fail (YELP_IS_WINDOW (window));
     yelp_window_load (window, "x-yelp-toc:");
 }
@@ -2540,7 +2750,7 @@ window_go_previous_cb (GtkAction *action, YelpWindow *window)
     YelpWindowPriv *priv;
     gchar *base, *uri;
 
-    d (g_print ("window_go_previous_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
     g_return_if_fail (YELP_IS_WINDOW (window));
     g_return_if_fail (window->priv->current_doc);
     priv = window->priv;
@@ -2560,7 +2770,7 @@ window_go_next_cb (GtkAction *action, YelpWindow *window)
     YelpWindowPriv *priv;
     gchar *base, *uri;
 
-    d (g_print ("window_go_next_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
     g_return_if_fail (YELP_IS_WINDOW (window));
     g_return_if_fail (window->priv->current_doc);
     priv = window->priv;
@@ -2580,7 +2790,7 @@ window_go_toc_cb (GtkAction *action, YelpWindow *window)
     YelpWindowPriv *priv;
     gchar *base, *uri;
 
-    d (g_print ("window_go_toc_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
     g_return_if_fail (YELP_IS_WINDOW (window));
     g_return_if_fail (window->priv->current_doc);
     priv = window->priv;
@@ -2600,7 +2810,7 @@ window_add_bookmark_cb (GtkAction *action, YelpWindow *window)
     gchar *uri;
     YelpWindowPriv *priv = window->priv;
 
-    d (g_print ("window_add_bookmark\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     uri = yelp_doc_info_get_uri (priv->current_doc, priv->current_frag,
 				 YELP_URI_TYPE_NO_FILE);
@@ -2662,32 +2872,12 @@ window_copy_mail_cb (GtkAction *action, YelpWindow *window)
 static void
 window_help_contents_cb (GtkAction *action, YelpWindow *window)
 {
-    GnomeProgram *program = gnome_program_get ();
-    GError *error = NULL;
-
-    g_return_if_fail (YELP_IS_WINDOW (window));
-
-    gnome_help_display_desktop (program, "user-guide", 
-                                "user-guide", "yelp", &error);
-
-    if (error) {
-	GtkWidget *dialog;
-
-	dialog = gtk_message_dialog_new (GTK_WINDOW (window),
- 	                                 0,
-	                                 GTK_MESSAGE_ERROR,
-	                                 GTK_BUTTONS_CLOSE,
-	                                 _("Could not display help for Yelp.\n"
-	                                 "%s"),
-	                                 error->message);
-
-	g_signal_connect_swapped (dialog, "response",
-	                          G_CALLBACK (gtk_widget_destroy),
-	                          dialog);
-	gtk_widget_show (dialog);
-
-	g_error_free (error);
-    }
+    /* Since we are already running, and are calling ourselves,
+     * we can make this function easy by just creating a new window
+     * and avoiding calling gnome_help_display_desktop
+     */
+    g_signal_emit (window, signals[NEW_WINDOW_REQUESTED], 0, 
+		   "ghelp:user-guide#yelp");
 }
 
 static void
@@ -2729,8 +2919,8 @@ location_response_cb (GtkDialog *dialog, gint id, YelpWindow *window)
 {
     g_return_if_fail (YELP_IS_WINDOW (window));
 
-    d (g_print ("location_response_cb\n"));
-    d (g_print ("  id = %i\n", id));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  id = %i\n", id);
 
     if (id == GTK_RESPONSE_OK) {
 	const gchar *uri = 
@@ -2778,12 +2968,6 @@ window_find_again (YelpWindow *window, gboolean forward)
 {
     YelpWindowPriv *priv = window->priv;
 
-#ifndef TYPEAHEADFIND
-    window_find_save_settings (window);
-
-    yelp_html_set_find_props (priv->html_view, priv->find_string, FALSE, FALSE);
-#endif /* TYPEAHEADFIND */
-
     return yelp_html_find_again (priv->html_view, forward);
 }
 
@@ -2899,14 +3083,14 @@ window_find_entry_key_pressed_cb (GtkWidget    *widget,
     YelpWindow     *window;
     YelpWindowPriv *priv;
 
+    debug_print (DB_FUNCTION, "entering\n");
+
     if (key->keyval == GDK_Escape) {
 	g_return_val_if_fail (YELP_IS_WINDOW (data), FALSE);
 
 	window = YELP_WINDOW (data);
 	priv = window->priv;
 
-	d (g_print ("window_find_entry_key_pressed_cb\n"));
-
 	gtk_widget_hide_all (priv->find_bar);
 
 	return TRUE;
@@ -2982,15 +3166,15 @@ idle_write (IdleWriterContext *context)
     g_return_val_if_fail (context != NULL, FALSE);
     g_return_val_if_fail (context->window != NULL, FALSE);
 
-    d (g_print ("idle_write\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     priv = context->window->priv;
 
     switch (context->type) {
     case IDLE_WRITER_MEMORY:
-	d (g_print ("  context->buffer = %i bytes\n", strlen (context->buffer)));
-	d (g_print ("  context->cur    = %i\n", context->cur));
-	d (g_print ("  context->length = %i\n", context->length));
+	debug_print (DB_DEBUG, "  context->buffer = %i bytes\n", strlen (context->buffer));
+	debug_print (DB_DEBUG, "  context->cur    = %i\n", context->cur);
+	debug_print (DB_DEBUG, "  context->length = %i\n", context->length);
 
 	if (context->cur + BUFFER_SIZE < context->length) {
 	    yelp_html_write (priv->html_view,
@@ -3013,7 +3197,7 @@ idle_write (IdleWriterContext *context)
     default:
 	g_assert_not_reached ();
     }
-
+    priv->idle_write = 0;
     return FALSE;
 }
 
